% Script to demonstrate the application of the AUTRES_* tools
%
% To run the demo in Matlab change to the folder in which this
% script is installed and enter AUTRES_Demo in response to the Matlab
% command prompt.
%
% AUTHOR
%
% David Vowles, Power Systems Dynamics Group,
% School of Electrical & Electronic Engineering,
% The University of Adelaide, Adealaide, South Australia
%
% David.Vowles@adelaide.edu.au
%
% DATE
% 
% 21/12/2013
%
%--------------------------------------------------------------------------------

figs = [];

fprintf(['\nExample 1: AUTRES_Load\n', ...
         'Load the time-responses contained in the files\n', ...
         '''.\\data\\IEEE_Case01_VrefStep_HPS_1_PSSE.csv'' and \n', ...
         '''.\\data\\IEEE_Case01_VrefStep_HPS_1_MP.csv''\n', ...
         'into the list of time-response data structures ''trlist''\n']);

PauseMessage;

filelist = { '.\data\IEEE_Case01_VrefStep_HPS_1_PSSE.csv', ...
             '.\data\IEEE_Case01_VrefStep_HPS_1_MP.csv' };

trlist = {};             
for i = 1:length(filelist)
   trlist{i} = AUTRES_Load(filelist{i});
end

PauseMessage;

fprintf(['\nExample 2: AUTRES_Extract\n', ...
         'Extract from the PSS/E (trlist{1}) and Mudpack (trlist{2})\n', ...
         'studies the response of the electrical power output (P)\n', ...
         'of the HPS_1 generator to a +0.5%% step-change in the AVR\n', ...
         'voltage-reference input of the generator.\n']);

t = {};
p = {};
for i = 1:length(trlist)
   [IsFound,t{i},p{i},chanName,chanUnits] = AUTRES_Extract(trlist{i},'P.HPS_1');
   if (not(IsFound))
      contents = AUTRES_Contents(trlist{i});
      error(['The variable ''P.HPS_1'' not found in the following ', ...
             'time-response\n', contents]);
   end          
end   

PauseMessage;

fprintf(['\nExample 3: Manual plotting of extracted variables\n', ...
         'Use Matlab''s built-in plotting functions to overlay the PSS/E\n', ...
         'and Mudpack responses of the power output of generator HPS_1\n', ...
         'to a +0.5%% step change in the AVR voltage-reference input\n']);

PauseFigure;

%Create a new figure
figs = [figs, figure()];

% Plot the PSS/E (t{1},p{1}) and Mudpack (t{2},p{2}) power responses
plot(t{1},p{1},'r',t{2},p{2},'b--');

% Set the time range from 0 to 5 s ... the PSS/E time vector starts at t = -2*DELT
% where DELT is the integration time-step.
set(gca,'xlim',[0, 5]);

% Add grid, title, axis labels and legend
% Set 'interpreter' to 'none' so that characters are interpretted as plain text and not
% according to their TeX or LaTeX meaning.
grid on
title({'Simplified 14-generator model of the South East Australian power system', ...
       'Case 1; Benchmark of PSS/E with Mudpack;', ...
       'Response of P.HPS_1 to 0.5% step on Vref.HPS_1'}, 'interpreter', 'none');
xlabel('Time (s)')
ylabel(['Perturbation in ' deblank(chanName) ' (' deblank(chanUnits) ')'],'interpreter','none');
legend('PSS/E', 'Mudpack', 'location', 'NorthEast')

PauseMessage;
       
fprintf(['\nExample 4: AUTRES_Plot\n', ...
         'Use the AUTRES_Plot function to produce the above plot with less coding\n']);

PauseFigure;

figs = [figs, figure()];
AUTRES_Plot(trlist,'P.HPS_1', ...
            'title', {'Simplified 14-generator model of the South East Australian power system', ...
                      'Case 1; Benchmark of PSS/E with Mudpack;', ...
                      'Response of P.HPS_1 to 0.5% step on Vref.HPS_1'});

PauseMessage;
       
fprintf(['\nExample 5: AUTRES_Load and AUTRES_Plot\n', ...
         'Use the AUTRES_Plot function to compare on a single plot the responses\n', ...
         'of the terminal voltages of the five SVCs following two-phase to ground\n', ...
         'faults at the 315 and 507 busesin case 2. The faults are cleared without\n', ...
         'disconnecting any network elements']);

filelist = { {'.\data\IEEE_NetworkBusFaultStudy_Case02_F315_NoTrip.mat', 'F315'}, ...
             {'.\data\IEEE_NetworkBusFaultStudy_Case02_F507_NoTrip.mat', 'F507'} };
studies = {};
for i = 1:length(filelist)
   studies{i} = AUTRES_Load(filelist{i}{1},filelist{i}{2});
end                
svcs = { 'ASVC_2', 'RSVC_3', 'BSVC_4', 'PSVC_5', 'SSVC_5' };
vars = {};
for i = 1:length(svcs)
   vars{i} = ['Vt.' svcs{i}];
end            

PauseFigure;

figs = [figs, figure()];
AUTRES_Plot(studies,vars, ...
            'title', {'Simplified 14-generator model of the South East Australian power system', ...
                      'Case 2; Compare SVC voltage responses for faults at buses 315 & 507'}, ...
            'ylabel', 'SVC voltage (pu)', ...
            'tmax',   5.0);

fprintf('\nPress ENTER to terminate the demo and delete the figures? ');
pause
fprintf('\n\n');
delete(figs);
