function timeResponseData = AUTRES_Load(filename,caseID)
% Load time response simulation data from native or csv file
%--------------------------------------------------------------------------------
%
% PURPOSE
%
% Load time response data that is stored in one of two file formats:
% (1) Native matlab (*.mat) format created with AUPlotData_AU2Mat
%     The file is expected to contain the following fields:
%     TT - time vector (npts x 1)
%     YY - channel data matrix (npts x nchan)
%     channelNames - cell array of channel names (1 x nchan)
%     channelUnits - cell array of channel units (1 x nchan)
%     Optionally it may also include:
%     CaseID       - case identification code
%     Notes        - cell array of notes describing the study case
%
% (2) CSV format
%     Row  1:            nn = number of lines of notes
%     Rows 2 to 1+nn:    notes describing the study case
%     Row  2+nn:         list of comma separated channel names
%     Rows 3+nn:         list of comma separated channel units
%     Rows 4+nn to end:  matrix of simulation results
%             the first column contains the time vector
%             columns 2 to end contain the channel data
%
% INPUTS
%
% filename  - name of the file containing the time response data
%             if the filename extension is '.mat' then the file is
%             assumed to be in native format; otherwise CSV format
%             is assumed.
%
% caseID    - (Optional) - Name to be assigned to the study case
%             Default: the base name of the file; or if the file is a native
%             file containing the caseID field then the value of caseID in the
%             file is used.
%
% OUTPUTS
%
% timeResponseData structure with the following components:
%    TT  - time vector (npts x 1)
%    YY  - channel data matrix (npts x nchan)
%          YY(i,j) contains the value of channel channelNames(j)
%          at time TT(i) in units of channelUnits(j).
%    channelNames - cell array of channel names (1 x nchan)
%    channelUnits - cell array of channel units (1 x nchan)
%    caseID       - study case identifier
%    Notes        - Notes describing the simulation from which the
%                   results are derived.
%
% AUTHOR
%
% David Vowles, Power Systems Dynamics Group,
% School of Electrical & Electronic Engineering,
% The University of Adelaide, Adealaide, South Australia
%
% David.Vowles@adelaide.edu.au
%
% DATE
% 
% 21/12/2013
%
%--------------------------------------------------------------------------------

   [filepath,filebase,fileext] = fileparts(filename);
   
   if (nargin < 2)
      caseID = filebase;
   end
      
   if (strcmpi(fileext,'.mat'))
      xx = load(filename);
      if (not(all(isfield(xx,{'TT','YY','channelNames','channelUnits'}))))
         error(['The Matlab native time response file ' filename, ...
                ' does not contain all of the required fields ', ...
                'TT, YY, channelNames, channelUnits']);
      end
      TT = xx.TT;
      YY = xx.YY;
      channelNames = xx.channelNames;
      channelUnits = xx.channelUnits;
      if (isfield(xx,'caseID'))
         if (nargin < 2)
            caseID = xx.caseID;
         end
      end      
      Notes = {};
      if (isfield(xx,'Notes'))
         Notes = xx.Notes;
      end            
   else
      % Open file and extract channel names from line 1 and channel units
      % from line 2;
      fid = fopen(filename,'r');
      nline = str2num(fgetl(fid));
      Notes = {};
      for i = 1:nline
         l = fgetl(fid);
         l = regexprep(l,'^[''"]','');
         l = regexprep(l,'[''"]$','');
         Notes{i} = l;
      end
      str = fgetl(fid);
      channelNames = strsplit(str,',');
      str = fgetl(fid);
      channelUnits = strsplit(str,',');
      fclose(fid);
      
      % Remove the header of the time column
      channelNames = channelNames(2:end);
      channelUnits = channelUnits(2:end);
      
      % Load the time response data
      xx = dlmread(filename,',',1 + nline + 2,0);
      TT = xx(:,1);
      YY = xx(:,2:end);
      clear('xx');
      
      % Get the number of columns
      nchan = min([length(channelNames),length(channelUnits),size(YY,2)]);
      
      % Trim number of channels ...
      channelNames = channelNames(1:nchan);
      channelUnits = channelUnits(1:nchan);
      YY = YY(:,1:nchan);
      
      % Remove quote characters from the channel names and units, if they exist
      for ichan = 1:nchan
         c = deblank(channelNames{ichan});
         u = deblank(channelUnits{ichan});
         c = regexprep(c,'^[''"]','');
         c = regexprep(c,'[''"]$','');
         u = regexprep(u,'^[''"]','');
         u = regexprep(u,'[''"]$','');
      end   
      
   end
   
   timeResponseData.TT           = TT;
   timeResponseData.YY           = YY;
   timeResponseData.channelNames = channelNames;
   timeResponseData.channelUnits = channelUnits;
   timeResponseData.caseID       = caseID;
   timeResponseData.Notes        = Notes;
   timeResponseData.filename     = filename;
   
   AUTRES_Contents(timeResponseData);
   
      