function AUTRES_Plot(timeResponseList,varList,varargin)
% Plot selected time response data
%--------------------------------------------------------------------------------
%
% PURPOSE
%
% Plot selected time responses in a straight-forward manner with a minimal user
% interface. Provided as a stand alone plotting program with the Simplified 14
% generator model of the SE Australian power system.
% 
% It is possible to overlay on a single plot responses of multiple variables
% from multiple simulations.
%
% INPUTS
%
% timeResponseList:
%    (1) a time response structure, usually obtained using AUTRES_Load; or
%    (2) a string containing the name of a time response file (the data
%        will be loaded from the file with AUTRES_Load); or
%    (3) A cell array of items in (1) and/or (2)
%
% varList - Either:
%    (1) a string containing the name of the channel to be plotted; or
%    (2) a cell array of such strings
%
% varargin - optional argments are entered in the following key, value pairs:
%
%    key    value
%    title  String or cell array of strings to be displayed as a plot title.
%           Default: no title is displayed.
%
%    xlabel String to be displayed as the x-label on the plot
%           Default: 'Time (s)'
%
%    ylabel String to be displayed as the y-label on the plot
%           Default: If varList contains only a single variable then
%           ylabel is the name of the variable and the associated units
%           in brackets; otherwise no ylabel is displayed.
%
%    legend Cell array of strings to be used as a plot legend.
%           legend{i,j} is the legend string for variable varList{j} 
%           from the response timeResponseDataList{i}
%           Default: If time response data from multiple studies is
%           displayed the legend for each response includes the
%           caseID of the associated time response. If more
%           than one variable is displayed the legend for each curve includes
%           the name of the variable.
%
%    tmin   Minimum time (s)
%
%    tmax   Maximum time (s)
%
%    ymin   Minimum value of y
%
%    ymax   Maximum value of y
%
% OUTPUTS
%
% None
%
%
% EXAMPLES
%
% % (1) Plot single variable (P.BPS_2) from single time response (Case01)
%
% Case01 = AUTRES_Load('Case01.csv');
% AUTRES_Plot(Case01,'P.BPS_2')
%
% % (2) Plot single variable (P.BPS_2) from multiple time responses
% %     (Case01 ... Case06)
%
% Cases  = {};
%
% for icase = 1:6
%    Cases{icase} = AUTRES_Load(['Case' sprintf('%02d',icase) '.csv']);
% end
%
% % Display with default settings of title, axis labels, etc.
% AUTRES_Plot(Cases,'P.BPS_2')
%
% % Display with customization of title, legend, ...
% AUTRES_Plot(Cases,'P.BPS_2', ...
%    'title',  { 'Simplified 14 generator model of the SE Australian system.', ...
%               ['Two-phase to ground fault on bus 206 cleared in 100 ms; ', ...
%                'P.BPS_2 for cases 1 to 6']}, ...
%    'legend', {'C1','C2','C3','C4','C5','C6'})
%
% % (3) For case 2 plot multiple variables (DEL.HPS_1, ..., DEL.PPS_5) for multiple 
% %     disturbances
%
% Cases  = {};
% Vars   = {};
%
% GenNameList = { 'HPS_1', 'BPS_2', 'EPS_2', 'VPS_2', 'MPS_2', ...
%                 'LPS_3', 'YPS_3', 'TPS_4', 'CPS_4', 'SPS_4', ...
%                 'GPS_4', 'NPS_5', 'TPS_5', 'PPS_5' };
%
% FaultBuses = { 102, 206, 209, 208, 215, 303, 312, 410, 408, 407, 405, ...
%                504, 505, 506 };
%
% for i = 1:length(FaultBuses)
%    Cases{i} = AUTRES_Load(['FaultStudy_Case02_' ...
%               '_' GenNameList{i} '_F' num2str(FaultBuses{i}) ...
%               '_NoTrip.csv']);
%    Vars{i}  = ['DEL.' GenNameList{i}]; 
% end
%
% AUTRES_Plot(Cases,Vars);
%
% AUTHOR
%
% David Vowles, Power Systems Dynamics Group,
% School of Electrical & Electronic Engineering,
% The University of Adelaide, Adealaide, South Australia
%
% David.Vowles@adelaide.edu.au
%
% DATE
% 
% 21/12/2013
%
%--------------------------------------------------------------------------------

   if (not(iscell(timeResponseList)))
      timeResponseList = { timeResponseList };
   end
   
   if (not(iscell(varList)))
      varList = { varList };
   end
         
   nres = length(timeResponseList);
   for ires = 1:nres
      tr = timeResponseList{ires};
      if (ischar(tr))
         timeResponseList{ires} = AUPlotData_LoadTimeResponse(tr);
      end         
   end

   nvar = length(varList);
   
   % Generate the default legend
   defaultLegend = {};
   for ires = 1:nres
      caseID = timeResponseList{ires}.caseID;
      for ivar = 1:nvar
         vnam = varList{ivar};

         lstr = '';
         if (nres > 1)
            lstr = caseID;
            if (nvar > 1)
               lstr = [lstr ': '];
            end   
         end
         
         if (nvar > 1)
            lstr = [lstr vnam];
         end
               
         defaultLegend{ires,ivar} = lstr;
      
      end
         
   end

   p = inputParser;
   p.addParamValue('title',      '',            @(x) validateattributes(x, {'char', 'cell'}, {}));
   p.addParamValue('xlabel',     'Time (s)',    @(x) validateattributes(x, {'char'}, {}));
   p.addParamValue('ylabel',     '',            @(x) validateattributes(x, {'char'}, {}));
   p.addParamValue('legend',     defaultLegend, @(x) validateattributes(x, {'char','cell'}, { '2d', 'size', [nres, nvar]}));
   p.addParamValue('tmin',       0,             @(x) validateattributes(x, {'double'}, {}));
   p.addParamValue('tmax',     NaN,             @(x) validateattributes(x, {'double'}, {}));
   p.addParamValue('ymin',     NaN,             @(x) validateattributes(x, {'double'}, {}));
   p.addParamValue('ymax',     NaN,             @(x) validateattributes(x, {'double'}, {}));
   
   p.parse(varargin{:});
   Opt = p.Results;

   % Transfer plot data to data structure convenient for plotting ...
   curves = {};
   LegendStrings = {};
   VarUnits = cell(1,nvar);
   for i = 1:nvar
      VarUnits{i} = 0;
   end
      
   icurve = 0;
   for ires = 1:nres
      for ivar = 1:nvar
         [IsFound,tt,yy,chanName,chanUnits] = ...
            AUTRES_Extract(timeResponseList{ires},varList{ivar});
         if (IsFound)
            icurve = icurve + 1;
            curves{icurve} = struct('TT',tt,'YY',yy, ...
                             'channelName', chanName, ...
                             'channelUnits',chanUnits);
            LegendStrings{icurve} = Opt.legend{ires,ivar};
            if (isnumeric(VarUnits{ivar}))
               VarUnits{ivar} = chanUnits;
            end
            if (not(strcmpi(VarUnits{ivar},chanUnits)))
               VarUnits{ivar} = '';
            end                       
         end                    
      end
   end
   
   ncurve = icurve;
   
   % Generate the default y-axis label and assign it to the
   % ylabel option if necessary 
   if (nvar > 1)
      defaultYlabel = '';
   else
      defaultYlabel = varList{1};
      if (not(isempty(VarUnits{1})))
         defaultYlabel = [ defaultYlabel ' (' VarUnits{1} ')' ];
      end       
   end
   
   if (isempty(Opt.ylabel))
      Opt.ylabel = defaultYlabel;
   end
      
   col = { 'r', 'b', 'g', 'c', 'm', 'k' };
   sty = { '-', '--', '-.' };
   mkr = { '',  'x',  'o', 's', '^', 'v', '*' };
   ncol = length(col);
   nsty = length(sty);
   nmkr = length(mkr);
   
   clf
   icol  = 1;
   isty  = 1;
   imkr  = 1;
   istrt = 1;
   H    = [];
   tmax = -Inf;
   tmin =  Inf;
   ymax = -Inf;
   ymin =  Inf;
   for icurve = 1:ncurve
      tt = curves{icurve}.TT;
      yy = curves{icurve}.YY;
      tmax = max([tmax,max(tt)]);
      tmin = min([tmin,min(tt)]);
      ymax = max([ymax,max(yy)]);
      ymin = min([ymin,min(yy)]);
      if (icol > ncol)
         icol = 1;
         isty = isty + 1;
      end
      if (isty > nsty)
         isty = 1;
         imkr = imkr + 1;
      end
      if (imkr > nmkr)
         imkr = 1;
      end
      c = col{icol};
      s = sty{isty};
      m = mkr{imkr};
      h = line(tt,yy,'color',c,'linestyle',s);
      if (imkr > 1)
         npts = length(tt);
         dn = ceil(npts/20);
         is = ceil(istrt/10*dn);
         istrt = istrt + 1;
         if (istrt > 10)
            istrt = 1;
         end   
         II = is:dn:npts;
         h = line(tt(II),yy(II),'color',c,'Marker',m,'linestyle','none');
         h = line(tt,yy,'color',c,'Marker',m,'linestyle',s,'visible','off');
      end
      H = [H; h];         
      icol = icol + 1;
   end
   
   if (ncurve > 1)
      if (not(isempty(LegendStrings)))
         legendPosition = 'NorthEast';
         if (ncurve > 6)
            legendPosition = 'EastOutside';
         end   
         legend(H,LegendStrings,'location',legendPosition,'interpreter','none');
      end
   end      

   if (not(isempty(Opt.title)))
      title(Opt.title,'interpreter','none');
   end
   
   if (not(isempty(Opt.ylabel)))
      ylabel(Opt.ylabel,'interpreter','none');
   end
   
   if (not(isempty(Opt.xlabel)))
      xlabel(Opt.xlabel,'interpreter','none');
   end
   
   xlim = get(gca,'xlim');
   ylim = get(gca,'ylim');
   xtick = get(gca,'xtick');
   ytick = get(gca,'ytick');
   
   xrange      = xlim(2)-xlim(1);
   norm_xrange = xrange/ceil(log10(xrange));
   min_xinc    = (norm_xrange/1e4)*ceil(log10(xrange));
   
    
   if (isnan(Opt.tmin))
      Opt.tmin = xlim(1);
      if ((tmin + min_xinc) > xtick(2))
         Opt.tmin = xtick(2)
      end   
   end
   xlim(1) = Opt.tmin;
      
   if (isnan(Opt.tmax))
      Opt.tmax = xlim(2);
      if ((tmax - min_xinc) < xtick(end-1))
         Opt.tmax = xtick(end-1)
      end   
   end
   xlim(2) = Opt.tmax;
      
   if (not(isnan(Opt.ymin)))
      ylim(1) = Opt.ymin;
   end
      
   if (not(isnan(Opt.ymax)))
      ylim(2) = Opt.ymax;
   end
   
   set(gca,'xlim',xlim);   
   set(gca,'ylim',ylim);   

   grid on
   box on
   
end
   